/* objective.cc
 *
 * Simple examples of how to wrap--or otherwise connect--an
 *   objective function so it can work with the DirectSearch
 *   class.
 *
 * Anne Shepherd, 8/2000 (Thanks to Chris Siefert for the EvalF
 *   function).  Revised, pls, 1/2001
 */

#include "objective.h" 
#include "vec.h"   
#include <math.h>
#include <iostream.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <stdio.h>

double EvalF(const Vector<double> &point, bool &success, void * VFname);


/* Here, we use fcn to wrap the user's 
 * objective function.  
 */
void fcn(long vars, Vector<double> &x, double & f, bool & flag, void* nothing)
{
     
  if (vars < 1) {
      cerr << "\nError: Dimension cannot be less than 1!!\n";
      exit(1);
  }
  f = mySin(x);
  flag = true;
  return;
}

void fcn_2(long vars, Vector<double> &x, double & f,
           bool & flag, void* nothing)
{
     
  if (vars < 1) {
      cerr << "\nError: Dimension cannot be less than 1!!\n";
      exit(1);
  }
  f = myCos(x);
  flag = true;
  return;
}

/**
 * This uses the EvalF() function to call an executable.  It gets its
 * "f" value from the stdout of the objective function.  See user's manual.
 * void * VFname is used to send in the name of the executable.
 */

void fcn_eval(long vars, Vector<double> &x, double & f, bool & flag,
              void * VFname)
{
    
     Vector<double> vec(x);
     f = EvalF(vec, flag, VFname);
}

void parab_fcn(long vars, Vector<double> & x, double & f,
               bool & flag, void * nothing)
{
  if (vars != 2) {
      cerr << "\nError: Dimension must be 2 !!\n";
      exit(1);
  }
  f = eval_parab(x);
  flag = true;
  return;
}

/* see K.I.M. McKinnon, ``Convergence of the Nelder-Mead Simplex
   Method to a Nonstationary Point,'' Siam J. Optim., Vol 9, No. 1,
   pp. 148-158. (1998)

   vars must equal 2

   Note that this function is not wrapped--I just wrote it to
   take the parameters required by the searches.    
*/
void mcKinnon(long vars, Vector<double> & x, double & f,
              bool & flag, void * nothing)
{
  
    double my_x = x[0];
    double my_y = x[1];
    if (my_x <= 0) {
        f = (360 * (my_x * my_x)) + my_y + (my_y * my_y);
    }
    else {
        f = (6 * (my_x * my_x)) + my_y + (my_y * my_y);
    }
    flag = true;
}

/** Powell's quartic function, as discussed in Nelder and Mead's 1965
 * paper.
 *
 *    y= (x_1 + 10*x_2)^2 + 5*(x_3 - x_4)^2
 *             + (x_2 - 2*x_3)^4 + 10*(x_1 - x_4)^4
 *
 *  Minimum value is 0. vars must equal 4;
 *
 *  Nelder and Mead tested this with starting point (3, -1, 0, 1).
 */
void powell(long vars, Vector<double> &x,
              double & f, bool & flag, void* nothing)
{
    if (vars != 4) {
        cerr << "\nThis function required four variables. ";
        cerr << "Exiting with value 1.\n";
        exit(1);
    }
    double my_x1 = x[0];
    double my_x2 = x[1];
    double my_x3 = x[2];
    double my_x4 = x[3];

    double first, second, third, fourth;

    first = (my_x1 + 10 * my_x2) * (my_x1 + 10 * my_x2);
    second = 5 * (my_x3 - my_x4) * (my_x3 - my_x4);
    third = (my_x2 - (2 * my_x3)) * (my_x2 - (2 * my_x3));
    third *= third;
    fourth = (my_x1 - my_x4) * (my_x1 - my_x4);
    fourth *= fourth;
    fourth *= 10;

    f = first + second + third + fourth;

    flag = true;
}

/*  We'll minimize the sine of a number--here vec1
 *  is a Vector of one member. (Or even if it isn't, we just
 *  take the sine of the first element.)
 */
double mySin(Vector<double> & vec1)
{
    double scalar = vec1.begin()[0];
    return sin(scalar);
}
    
/*
 * and here we find the cosine
 */
double myCos(Vector<double> & vec2)
{
    double scalar = vec2.begin()[0];
    return cos(scalar);
}

/*
 * here, it's an elliptical paraboloid--min at
 * the origin. n must equal 2.
 */
double eval_parab(Vector<double> & parab_vec)
{
    double my_x = parab_vec[0];
    double my_y = parab_vec[1];
    
    return ( (my_x * my_x) + ( 3 * (my_y * my_y) ) );
}


#define ERROR HUGE_VAL

/****
 * Evalf()
 *   written by Chris Siefert, slightly modified by Anne Shepherd
 *    
 * INPUT: The point at which to evaluate the function; a boolean flag;
 *     the file name.
 * OUTPUT: The function value.
 * EFFECT: This function will do the fork/exec dance, sending the
 *     program the parameters on its stdin, then after waiting, it will
 *     get a  double from the  program's stdout.  If the program exited
 *     with an error, it will return ERROR and set success to false.
 *     Otherwise success will be true.
 ****/

double EvalF(const Vector<double> &point, bool &success, void * VFname) {
  int childpid, pipe1[2], pipe2[2];
  const char* Fname;
  long i;
  long P = point.dim();
  double ret_val;
  if (VFname != NULL) {
     Fname = (const char *) VFname;
  }
  FILE *readpipe;
  FILE *writepipe;
  
  long state;
  
  if ((pipe(pipe1) < 0) || (pipe(pipe2) < 0) ) {
    perror("pipe");
    exit(-1);
  }/*end if*/

  if ((childpid = fork()) < 0) {
    perror("fork");
    exit(-1);
  } else if (childpid > 0) {   /*Parent*/
    close(pipe1[0]); close(pipe2[1]);
    
    /* Write to child on pipe1[1], read from child on pipe2[0]. */
    readpipe=fdopen(pipe2[0],"r");
    writepipe=fdopen(pipe1[1],"w");
    if(writepipe==NULL || readpipe==NULL)
      {perror("fdopen");cerr<<"MPEF   : Cannot open pipes... exiting\n";abort();}
   
    /*Output the vector x*/
    for(i=0;i<P;i++) fprintf(writepipe,"%1.8f ", point[i]); 
    fprintf(writepipe,"\n");
    fflush(writepipe);

    /*Read in the returned value, and WAIT*/
    fscanf(readpipe,"%lf", &ret_val);    
    wait(&state);
    /*Close outstanding pipes*/
    fclose(writepipe);fclose(readpipe);
    
    if(WIFEXITED(state)&&(WEXITSTATUS(state)==0)) {
      /*normal exit - no abort, return code 0*/
      success=true;
      return ret_val;     
    }/*end if*/
    else {
      success=false;
      success = true;
      return ERROR;
    }/*end else*/
    
  } else { /*Child*/
    close(pipe1[1]); close(pipe2[0]);
    /* Read from parent on pipe1[0], write to parent on pipe2[1]. */
    dup2(pipe1[0],0); dup2(pipe2[1],1);
    close(pipe1[0]); close(pipe2[1]);

    if (execlp(Fname, Fname, NULL) < 0) {
      perror("execlp");
      abort();
    }
    return ERROR;
    /* Never returns */
  }/*end else*/

  return ERROR;  /*never returns*/
}/*end EvalF*/







